(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     28208,        937]*)
(*NotebookOutlinePosition[     28857,        960]*)
(*  CellTagsIndexPosition[     28813,        956]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA - 1", "Title"],

Cell["Generating the Primes", "Subtitle"],

Cell["Let's import a package that will help to generate the key.", "Text"],

Cell[BoxData[
    \(<< NumberTheory`PrimeQ`\)], "Input"],

Cell["\<\
The algorithm we shall use will be to choose a large random number of a \
similar size to our desired prime.  We then look for the next prime number to \
follow.  Bressoud (ch. 2) explains how there are a lot of primes distriubted \
through the integers and explains that the technique we will use is a \
reasonable way to find large primes.\
\>", "Text"],

Cell[TextData[{
  "We use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s speudo-random number generator."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(rand = Random[Integer, {10\^10, 10\^11}]\)], "Input"],

Cell[BoxData[
    \(90515782875\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Here we have specified that we want an Integer in the given range, ",
  Cell[BoxData[
      \(TraditionalForm\`10\^10\)]],
  " to ",
  Cell[BoxData[
      \(TraditionalForm\`10\^11\)]],
  "."
}], "Text"],

Cell["\<\
The next step is to get a random number following this number.  Bressoud \
suggests ignoring all multiples of 2 and 3 which make up a third of the \
numbers and can never be prime.  However, since we onl expect to do a \
reasonably small number of checks, we will just ignore the multiples of 2.\
\>", "Text"],

Cell[BoxData[
    \(GetNextPrime[num_] := 
      Module[{n}, \n\t\tIf[EvenQ[num], n = num + 1, n = num]; \n\t\t
        While[\(! PrimeQ[n]\), n = n + 2]; \n\t\tn]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetNextPrime[rand]\)], "Input"],

Cell[BoxData[
    \(90515782891\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(PrimeQ[%]\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell["\<\
The algorithm first checks to see if the number is even, and if it is it adds \
1 to make it odd.  Then we go through all the odd numbers until a prime is \
found.\
\>", "Text"],

Cell["\<\
The next step is to wrap these steps into a single function.  The function \
should take as a parameter an integer that specifies the approximate order of \
the prime required.  The integer will represent the power of 10 about which a \
random number should be chosen.  There is an optional scope argument that \
allows the range to be broadened about the range.\
\>", "Text"],

Cell[BoxData[
    \(GenerateRandomPrime[n_] := 
      GetNextPrime[Random[Integer, {10\^\(n - 1\), 10\^n}]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[GenerateRandomPrime[300]]\)], "Input"],

Cell[BoxData[
    \({1.69999999999999928`\ Second, 
      222043728519982800500351829603091487264199409292342342497788715473387938\
058287112237666147343025540779049305197278032017378737081984205688982848662781\
013233136162278335519766343203265736266149620280551011937428391317496973500947\
319706229148704281434174362303431143909690388242354553391304677784692477}\)], 
  "Output"]
}, Open  ]],

Cell["\<\
As can be seen we can quickly generate primes with this method.\
\>", "Text"],

Cell[CellGroupData[{

Cell["What about strong primes?", "Subtitle"],

Cell["\<\
A \"strong\" prime is called so because it has properties that will make it \
more difficult to factor a product of two such strong primes. The original \
RSA paper suggested their use to avoid factorizing methods such as the \
Pollard p-1 approach. \
\>", "Text"],

Cell["\<\
Note that they may be obsolete as Rivest and Silverman (Are 'Strong' Primes \
Needed for RSA) have suggested.\
\>", "Text"],

Cell["\<\
The first step is to generate a set of random primes. We call these p''. From \
these we will generate primes, p', such that p'= 2*p'' + 1\
\>", "Text"],

Cell[CellGroupData[{

Cell[TextData[{
  "Finding ",
  Cell[BoxData[
      \(TraditionalForm\`\(p\^--\)\)]],
  " strong primes"
}], "Subsubtitle"],

Cell[BoxData[
    \(GetPmmStrong[ord_] := \n\t
      Module[{pdd, pd}, \n\t\tpdd = GenerateRandomPrime[ord - 2]; \n\t\t
        pd = Strengthen[pdd]; \n\t\tStrengthen[pd]]\)], "Input"],

Cell[BoxData[
    \(Strengthen[pd_, fun_:  Plus] := 
      Module[{p, ad}, \n\t\tad = 2; \n\t\tp = fun[ad*pd, 1]; \n\t\t
        While[\[Not] PrimeQ[p], \n\t\t\tad = ad + 2; \n\t\t\t\t
          p = fun[ad*pd, 1]]; \n\t\tp]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p = GetPmmStrong[10]\)], "Input"],

Cell[BoxData[
    \(27595201187\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorInteger[p - 1]\)], "Input"],

Cell[BoxData[
    \({{2, 2}, {1440947887, 1}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pp = Strengthen[p, Subtract]\)], "Input"],

Cell[BoxData[
    \(110380804747\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorInteger[pp + 1]\)], "Input"],

Cell[BoxData[
    \({{2, 2}, {27595201187, 1}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorInteger[pp - 1]\)], "Input"],

Cell[BoxData[
    \({{2, 1}, {3, 1}, {38321, 1}, {480071, 1}}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Finding Strong Primes", "Subsubtitle"],

Cell[TextData[{
  "Strong primes should be ",
  Cell[BoxData[
      \(TraditionalForm\`\(p\^+\)\)]],
  " strong as well as ",
  Cell[BoxData[
      \(TraditionalForm\`\(p\^--\)\)]],
  " strong. This algorithm is due to Williams.Schmid as I found it in \
Rivest's paper ",
  StyleBox["Are \"Strong\" Primes needed...",
    FontSlant->"Italic"]
}], "Text"],

Cell[BoxData[
    \(GetStrong[ord_, prn_:  False] := \n\t
      Module[{pmm, pp, r, g, k, pm, a, p}, \n\t\t
        pmm = GenerateRandomPrime[ord/2 - 1]; \n\t\t
        pp = GenerateRandomPrime[ord/2 - 1]; \n
        \t\t{g, {r, k}} = ExtendedGCD[pmm, pp]; \n\t\tr = \(-\((r)\)\); \n\t\t
        a = 1; pm = 4; p = 4; \n\t\t
        While[\[Not] \((PrimeQ[pm] \[And] PrimeQ[p])\), \n\t\t\t
          pm = \((2*a*pmm*pp)\) + \((2*r*pmm)\) + 1; \n\t\t\t
          p = \((2*pm)\) + 1; \n\t\t\t\(a++\)]; \n\t\t
        If[prn, \n\t\t\t
          Print[\*"\"\<\!\(\(P\^+\)\):\>\"", pp, 
            \*"\"\<,\!\(\(p\^--\)\):\>\"", pmm, \*"\"\<,\!\(\(p\^-\)\):\>\"", 
            pm]]; \n\t\tp]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p = GetStrong[20, True]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      \("\!\(\(P\^+\)\):"\[InvisibleSpace]355415471
        \[InvisibleSpace]",\!\(\(p\^--\)\):"\[InvisibleSpace]922883141
        \[InvisibleSpace]",\!\(\(p\^-\)\):"\[InvisibleSpace]\
195869677370416649861\),
      SequenceForm[ 
      "\!\(\(P\^+\)\):", 355415471, ",\!\(\(p\^--\)\):", 922883141, 
        ",\!\(\(p\^-\)\):", 195869677370416649861],
      Editable->False]], "Print"],

Cell[BoxData[
    \(391739354740833299723\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[p]]\)], "Input"],

Cell[BoxData[
    \(21\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorInteger[p + 1]\)], "Input"],

Cell[BoxData[
    \({{2, 2}, {3, 1}, {23, 1}, {107, 1}, {37322267, 1}, {355415471, 1}}\)], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorInteger[p - 1]\)], "Input"],

Cell[BoxData[
    \({{2, 1}, {195869677370416649861, 1}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorInteger[195869677370416649861 - 1]\)], "Input"],

Cell[BoxData[
    \({{2, 2}, {5, 1}, {13, 1}, {449, 1}, {761, 1}, {2389, 1}, {922883141, 1}}
      \)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Timing", "Subtitle"],

Cell["\<\
Let's have a look at the time it takes to generate various strong primes.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[p1 = GetStrong[100]]\)], "Input"],

Cell[BoxData[
    \({116.829999999999922`\ Second, 
      631108250526242000475986982236755096815513876750346273173669372655343591\
7876889373641048375146513096023}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[p1]]\)], "Input"],

Cell[BoxData[
    \(103\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "What about the ",
  Cell[BoxData[
      \(TraditionalForm\`\(p\^--\)\)]],
  " strong primes"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[p2 = GetPmmStrong[100]]\)], "Input"],

Cell[BoxData[
    \({2.57999999999992724`\ Second, 
      126005549812976391305992946888180304975708170614838754525589995818002138\
49827651199033582880202122759379}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[p2]]\)], "Input"],

Cell[BoxData[
    \(104\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Wow! That is considerably faster. This is because we have to search \
through a much greater number of ",
  StyleBox["a",
    FontSlant->"Italic"],
  " values in the GetStrong method. The calculations are also a bit more \
complex. What about trying 512 bit numbers. This is about 170 decimal \
digits."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[p3 = GetPmmStrong[170]]\)], "Input"],

Cell[BoxData[
    \({15.9300000000002928`\ Second, 
      112758467066045874660297736840685708167064361412460433383374031535692273\
416169990658887917944614217902394690863188900093784597711704282934129729240285\
288438810544640457237747}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[p3, 2]]\)], "Input"],

Cell[BoxData[
    \(575\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Which to use", "Subtitle"],

Cell[TextData[{
  "By rights I should only use the Strong prime algorithm, but it is slow. I \
will therefore by default use the ",
  Cell[BoxData[
      \(TraditionalForm\`\(p\^--\)\)]],
  " algorithm and allow the other to be specified through optional \
parameters."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[GenerateStrongPrime] = {Algorithm \[Rule] GetPmmStrong}\)], 
  "Input"],

Cell[BoxData[
    \({Algorithm \[Rule] GetPmmStrong}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[GenerateStrongPrime]\)], "Input"],

Cell[BoxData[
    \({Algorithm \[Rule] GetPmmStrong}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(GenerateStrongPrime[ord_] = GenerateStrongPrime[ord, {}]; \n
    GenerateStrongPrime[ord_, opts__] := 
      Module[{alg}, \n\t\t
        alg = \(Algorithm /. opts\) /. Options[GenerateStrongPrime]; \n\t\t
        alg[ord]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[GenerateStrongPrime[50, Algorithm \[Rule] GetStrong]]\)], "Input"],

Cell[BoxData[
    \({12.8500000000003655`\ Second, 
      8974510985090812349931671126431293339653871669580139}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[GenerateStrongPrime[50]]\)], "Input"],

Cell[BoxData[
    \({0.169999999999163264`\ Second, 
      200962635259837357708998695815243418868360468004067}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Other stuff", "Subtitle"],

Cell["\<\
Here is some older work that I did before I tried the above algorithms.\
\>", "Text"],

Cell[BoxData[
    \(\(pdd = Table[GenerateRandomPrime[20], {x, 1, 100}]; \)\)], "Input"],

Cell[TextData[
"We want to go through this array, mapping p''\[Rule] 2*p'' + 1. Then we \
select the primes from this list. Let's write a function for it."], "Text"],

Cell[BoxData[
    \(GetStrong[l_] := 
      Module[{temp}, \[IndentingNewLine]temp = Map[\((\((2*#)\) + 1)\)&, l]; 
        \[IndentingNewLine]Select[temp, PrimeQ]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pd = GetStrong[pdd]\)], "Input"],

Cell[BoxData[
    \({556273863437379662423, 916758752848188778679}\)], "Output"]
}, Open  ]],

Cell["\<\
We were quite unlucky with the attempt.  We can improve this by increasing \
the number of original primes selected.\
\>", "Text"],

Cell[BoxData[
    \(\(pdd2 = Table[GenerateRandomPrime[20, 1], {x, 1, 2000}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pd2 = GetStrong[pdd2]\)], "Input"],

Cell[BoxData[
    \({465409861376150227247, 1898593202633920035203, 1382145486451165254587, 
      1431294507294095315519, 973659530181300792587, 670107800185210816763, 
      346813739289883842827, 1535026677358328116199, 922770317079528192767, 
      1829056816261132412399, 1547139064640860659407, 1647562829043379667183, 
      413598327400050867983, 727641974843073758927, 1016281590868346349887, 
      1958603159534010217043, 106533511472161011299, 656608302422661159119, 
      244212647112532804919, 1247155413698634657359, 1813489401958198893647, 
      969157761741161409203, 31478265947658369023, 472604666124488024627, 
      364291480426313117879, 1538431514548218375539, 108881512743988625099, 
      1354771043511227457827, 1856388498960915799787, 220927656163145867939, 
      1832574521347619010623, 1382537872621733277587, 665320067364994857227, 
      1145798768734716673307, 1679103596321565297083, 1671267415707843219479, 
      959446908088078944299, 1378919314127188133219, 1651152945617316536807, 
      108711584061651380879, 1804889838839778501263, 1689909178465059716327, 
      565759065992070314447, 262312818057941705963, 1308256147827072103403, 
      1637797466602894535123, 71738449996591339799, 1512559035115997535323, 
      1299594711937516791803, 1630868185931771453999, 611901442931092883243, 
      514123022077222638287, 1919116669723196844179, 601402008055989905327, 
      295946386552072570499, 327456653668595724047, 1952623007120915796419, 
      537437547731419236179, 573037929530578023839}\)], "Output"]
}, Open  ]],

Cell["Much better. But let's pass the GetStrong method again.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p = GetStrong[pd2]\)], "Input"],

Cell[BoxData[
    \({3797186405267840070407, 3665149042695238021247, 
      3025118070231995070647}\)], "Output"]
}, Open  ]],

Cell["Now we have candidates for p and q that are quite strong.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(n = 
      p\[LeftDoubleBracket]1\[RightDoubleBracket]*
        p\[LeftDoubleBracket]2\[RightDoubleBracket]\)], "Input"],

Cell[BoxData[
    \(13917254118202796150165850747689419841937529\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[n, 2]]\)], "Input"],

Cell[BoxData[
    \(144\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorIntegerECM[n]\)], "Input"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Open  ]],

Cell["\<\
That's stupid. Let's just try an entire function that will generate a list of \
primes of given number of bits (e.g. 20) and then use the GetStrong methof \
twice to return a list of numbers that are strong primes.  I will place \
several print-outs that report on the progress of the fucntion.\
\>", "Text"],

Cell[BoxData[
    \(GetStrongPrimes[size_, InitialPrimes_:  2000] := 
      Module[{pdd, pd, p}, \[IndentingNewLine]Print["\<Generating \>", 
          InitialPrimes, "\< primes.\>"]; \[IndentingNewLine]pdd = 
          Table[GenerateRandomPrime[size, 1], {x, 1, 
              InitialPrimes}]; \[IndentingNewLine]Print["\<Finished List of \
Primes\>"]; \[IndentingNewLine]pd = 
          GetStrong[
            pdd]; \[IndentingNewLine]Print["\<First pass complete. no. of \
primes=\>", Length[pd]]; \[IndentingNewLine]p = GetStrong[pd]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetStrongPrimes[20]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Generating "\[InvisibleSpace]2000\[InvisibleSpace]" \
primes."\),
      SequenceForm[ "Generating ", 2000, " primes."],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Finished List of Primes"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("First pass complete. no. of \
primes="\[InvisibleSpace]62\),
      SequenceForm[ "First pass complete. no. of primes=", 62],
      Editable->False]], "Print"],

Cell[BoxData[
    \({180118011623224352207, 138476128228539276599, 
      1179019783656055146119}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ps = %\)], "Input"],

Cell[BoxData[
    \({180118011623224352207, 138476128228539276599, 
      1179019783656055146119}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(n = 
      ps\[LeftDoubleBracket]1\[RightDoubleBracket]*
        ps\[LeftDoubleBracket]2\[RightDoubleBracket]\)], "Input"],

Cell[BoxData[
    \(24942044873807143247755581697268169103993\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[n, 2]]\)], "Input"],

Cell[BoxData[
    \(135\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[GetStrongPrimes[30]]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Generating "\[InvisibleSpace]2000\[InvisibleSpace]" \
primes."\),
      SequenceForm[ "Generating ", 2000, " primes."],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Finished List of Primes"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("First pass complete. no. of \
primes="\[InvisibleSpace]39\),
      SequenceForm[ "First pass complete. no. of primes=", 39],
      Editable->False]], "Print"],

Cell[BoxData[
    \({54.158000000000015`\ Second, {31324517759320448027822321503799, 
        28191933189450095674706169120767}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Flatten[%]\)], "Input"],

Cell[BoxData[
    \({54.158000000000015`\ Second, 31324517759320448027822321503799, 
      28191933189450095674706169120767}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(n = %\[LeftDoubleBracket]2\[RightDoubleBracket]*%\[LeftDoubleBracket]3\
\[RightDoubleBracket]\)], "Input"],

Cell[BoxData[
    \(883098711862505082797640414999593673937719912395560537080293833\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[n, 2]]\)], "Input"],

Cell[BoxData[
    \(210\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[31324517759320448027822321503799, 2]]\)], "Input"],

Cell[BoxData[
    \(105\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[28191933189450095674706169120767, 2]]\)], "Input"],

Cell[BoxData[
    \(105\)], "Output"]
}, Open  ]],

Cell["\<\
Although we are getting decent results, generating enormous strong primes (we \
want ones about the 1024 bits) will be very time consuming.  We might try \
improving the screening process of GetStrong.  We could try using multiples \
of 3 to 10 as well as 2. This would give a larger selection for the PrimeQ \
test and still maintains the strength of the prime.\
\>", "Text"],

Cell[BoxData[
    \(GetStrong2[l_] := 
      Module[{temp}, \[IndentingNewLine]temp = 
          Map[\((\((2*#)\) + 1)\) &, l]; \[IndentingNewLine]temp = 
          Union[temp, 
            Map[\((\((3*#)\) + 1)\) &, l]]; \[IndentingNewLine]temp = 
          Union[temp, 
            Map[\((\((4*#)\) + 1)\) &, l]]; \[IndentingNewLine]temp = 
          Union[temp, 
            Map[\((\((5*#)\) + 1)\) &, l]]; \[IndentingNewLine]Select[temp, 
          PrimeQ]]\)], "Input"],

Cell[BoxData[
    \(GetStrong2[l_] := 
      Module[{temp}, \[IndentingNewLine]temp = 
          Map[\((\((2*#)\) + 1)\) &, l]; \[IndentingNewLine]For[i = 3, 
          i < 10, i = \(i++\), \[IndentingNewLine]temp = 
            Union[temp, 
              Map[\((\((i*#)\) + 1)\) &, l]]]; \[IndentingNewLine]Select[
          temp, PrimeQ]]\)], "Input"],

Cell[BoxData[
    \(GetStrongPrimes2[size_, InitialPrimes_:  2000] := 
      Module[{pdd, pd, p}, \[IndentingNewLine]Print["\<Generating \>", 
          InitialPrimes, "\< primes.\>"]; \[IndentingNewLine]pdd = 
          Table[GenerateRandomPrime[size, 1], {x, 1, 
              InitialPrimes}]; \[IndentingNewLine]Print["\<Finished List of \
Primes\>"]; \[IndentingNewLine]pd = 
          GetStrong2[
            pdd]; \[IndentingNewLine]Print["\<First pass complete. no. of \
primes=\>", Length[pd]]; \[IndentingNewLine]p = GetStrong2[pd]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[GetStrongPrimes2[35]]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Generating "\[InvisibleSpace]2000\[InvisibleSpace]" \
primes."\),
      SequenceForm[ "Generating ", 2000, " primes."],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Finished List of Primes"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("First pass complete. no. of \
primes="\[InvisibleSpace]67\),
      SequenceForm[ "First pass complete. no. of primes=", 67],
      Editable->False]], "Print"],

Cell[BoxData[
    \({67.89699999999999`\ Second, {149115528777338733623816421434157599, 
        211215823497615895211069690419525979, 
        315140001207297048011401511569877399}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[741822538475273652496142700209131739]]\)], "Input"],

Cell[BoxData[
    \(36\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[GetStrongPrimes2[35]]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Generating "\[InvisibleSpace]2000\[InvisibleSpace]" \
primes."\),
      SequenceForm[ "Generating ", 2000, " primes."],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Finished List of Primes"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("First pass complete. no. of \
primes="\[InvisibleSpace]72\),
      SequenceForm[ "First pass complete. no. of primes=", 72],
      Editable->False]], "Print"],

Cell[BoxData[
    \({69.07`\ Second, {96875024025164106656711546062201127}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(GetNPrimes[num_, n_] := 
      Module[{ps, i}, \[IndentingNewLine]ps = {}; \[IndentingNewLine]If[
          EvenQ[num], i = num + 1, i = num]; \[IndentingNewLine]While[
          Length[ps] < 
            n, \[IndentingNewLine]\(\(If[PrimeQ[i], 
              ps = Union[ps, {i}]];\)\[IndentingNewLine]
          i = i + 2\)]; \[IndentingNewLine]ps]\)], "Input"],

Cell[BoxData[
    \(GenerateRandomPrimeList[order_, size_] := 
      GetNPrimes[Random[Integer, {2\^\(order - 1\), 2\^order}], 
        size]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GenerateRandomPrimeList[100, 10]\)], "Input"],

Cell[BoxData[
    \({949726410206988356036011093751, 949726410206988356036011093829, 
      949726410206988356036011093853, 949726410206988356036011093871, 
      949726410206988356036011093993, 949726410206988356036011094017, 
      949726410206988356036011094081, 949726410206988356036011094153, 
      949726410206988356036011094227, 
      949726410206988356036011094291}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[GenerateRandomPrimeList[1024, 10]]\)], "Input"],

Cell[BoxData[
    \({72.72500000000002`\ Second, \
{17761457741033570805497275832072679450836110162082416728387306918656092864922\
446135036615493742819225762948590082095625870439747369769317982213942501361004\
900553464308481682604147123474818234745704913807404981913564508022002489309685\
9993460275150405899811388145558652992357742112146767458292422341848159567787, 
        1776145774103357080549727583207267945083611016208241672838730691865609\
286492244613503661549374281922576294859008209562587043974736976931798221394250\
136100490055346430848168260414712347481823474570491380740498191356450802200248\
930968599934602751504058998113881455586529923577421121467674582924223418481595\
67883, 17761457741033570805497275832072679450836110162082416728387306918656092\
864922446135036615493742819225762948590082095625870439747369769317982213942501\
361004900553464308481682604147123474818234745704913807404981913564508022002489\
309685999346027515040589981138814555865299235774211214676745829242234184815956\
7933, 177614577410335708054972758320726794508361101620824167283873069186560928\
649224461350366154937428192257629485900820956258704397473697693179822139425013\
610049005534643084816826041471234748182347457049138074049819135645080220024893\
096859993460275150405899811388145558652992357742112146767458292422341848159568\
329, 1776145774103357080549727583207267945083611016208241672838730691865609286\
492244613503661549374281922576294859008209562587043974736976931798221394250136\
100490055346430848168260414712347481823474570491380740498191356450802200248930\
968599934602751504058998113881455586529923577421121467674582924223418481595694\
21, 17761457741033570805497275832072679450836110162082416728387306918656092864\
922446135036615493742819225762948590082095625870439747369769317982213942501361\
004900553464308481682604147123474818234745704913807404981913564508022002489309\
685999346027515040589981138814555865299235774211214676745829242234184815956962\
3, 177614577410335708054972758320726794508361101620824167283873069186560928649\
224461350366154937428192257629485900820956258704397473697693179822139425013610\
049005534643084816826041471234748182347457049138074049819135645080220024893096\
859993460275150405899811388145558652992357742112146767458292422341848159570027\
, 1776145774103357080549727583207267945083611016208241672838730691865609286492\
244613503661549374281922576294859008209562587043974736976931798221394250136100\
490055346430848168260414712347481823474570491380740498191356450802200248930968\
59993460275150405899811388145558652992357742112146767458292422341848159570919,\
 17761457741033570805497275832072679450836110162082416728387306918656092864922\
446135036615493742819225762948590082095625870439747369769317982213942501361004\
900553464308481682604147123474818234745704913807404981913564508022002489309685\
9993460275150405899811388145558652992357742112146767458292422341848159571033, 
        1776145774103357080549727583207267945083611016208241672838730691865609\
286492244613503661549374281922576294859008209562587043974736976931798221394250\
136100490055346430848168260414712347481823474570491380740498191356450802200248\
930968599934602751504058998113881455586529923577421121467674582924223418481595\
72109}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetStrongPrimes2[10]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Generating "\[InvisibleSpace]2000\[InvisibleSpace]" \
primes."\),
      SequenceForm[ "Generating ", 2000, " primes."],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Finished List of Primes"\)], "Print"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{507, 655},
WindowMargins->{{2, Automatic}, {Automatic, 1}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 24, 0, 105, "Title"],
Cell[1758, 53, 41, 0, 64, "Subtitle"],
Cell[1802, 55, 74, 0, 33, "Text"],
Cell[1879, 57, 56, 1, 30, "Input"],
Cell[1938, 60, 365, 6, 90, "Text"],
Cell[2306, 68, 131, 5, 33, "Text"],

Cell[CellGroupData[{
Cell[2462, 77, 73, 1, 31, "Input"],
Cell[2538, 80, 45, 1, 29, "Output"]
}, Open  ]],
Cell[2598, 84, 222, 8, 33, "Text"],
Cell[2823, 94, 319, 5, 90, "Text"],
Cell[3145, 101, 176, 3, 90, "Input"],

Cell[CellGroupData[{
Cell[3346, 108, 51, 1, 30, "Input"],
Cell[3400, 111, 45, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3482, 117, 42, 1, 30, "Input"],
Cell[3527, 120, 38, 1, 29, "Output"]
}, Open  ]],
Cell[3580, 124, 187, 4, 52, "Text"],
Cell[3770, 130, 386, 6, 90, "Text"],
Cell[4159, 138, 120, 2, 51, "Input"],

Cell[CellGroupData[{
Cell[4304, 144, 65, 1, 30, "Input"],
Cell[4372, 147, 382, 6, 143, "Output"]
}, Open  ]],
Cell[4769, 156, 87, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[4881, 162, 45, 0, 64, "Subtitle"],
Cell[4929, 164, 274, 5, 71, "Text"],
Cell[5206, 171, 133, 3, 52, "Text"],
Cell[5342, 176, 162, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[5529, 183, 123, 5, 53, "Subsubtitle"],
Cell[5655, 190, 184, 3, 110, "Input"],
Cell[5842, 195, 236, 4, 150, "Input"],

Cell[CellGroupData[{
Cell[6103, 203, 53, 1, 30, "Input"],
Cell[6159, 206, 45, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6241, 212, 53, 1, 30, "Input"],
Cell[6297, 215, 59, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6393, 221, 61, 1, 30, "Input"],
Cell[6457, 224, 46, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6540, 230, 54, 1, 30, "Input"],
Cell[6597, 233, 60, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6694, 239, 54, 1, 30, "Input"],
Cell[6751, 242, 75, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6875, 249, 44, 0, 53, "Subsubtitle"],
Cell[6922, 251, 354, 11, 52, "Text"],
Cell[7279, 264, 700, 13, 290, "Input"],

Cell[CellGroupData[{
Cell[8004, 281, 56, 1, 30, "Input"],
Cell[8063, 284, 422, 9, 25, "Print"],
Cell[8488, 295, 55, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8580, 301, 57, 1, 30, "Input"],
Cell[8640, 304, 36, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8713, 310, 53, 1, 30, "Input"],
Cell[8769, 313, 103, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8909, 320, 53, 1, 30, "Input"],
Cell[8965, 323, 70, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9072, 329, 73, 1, 30, "Input"],
Cell[9148, 332, 113, 2, 48, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9322, 341, 26, 0, 64, "Subtitle"],
Cell[9351, 343, 97, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[9473, 349, 60, 1, 30, "Input"],
Cell[9536, 352, 178, 3, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9751, 360, 58, 1, 30, "Input"],
Cell[9812, 363, 37, 1, 29, "Output"]
}, Open  ]],
Cell[9864, 367, 123, 5, 33, "Text"],

Cell[CellGroupData[{
Cell[10012, 376, 63, 1, 30, "Input"],
Cell[10078, 379, 179, 3, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10294, 387, 58, 1, 30, "Input"],
Cell[10355, 390, 37, 1, 29, "Output"]
}, Open  ]],
Cell[10407, 394, 334, 8, 71, "Text"],

Cell[CellGroupData[{
Cell[10766, 406, 63, 1, 30, "Input"],
Cell[10832, 409, 251, 4, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11120, 418, 61, 1, 30, "Input"],
Cell[11184, 421, 37, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11270, 428, 32, 0, 64, "Subtitle"],
Cell[11305, 430, 281, 7, 52, "Text"],

Cell[CellGroupData[{
Cell[11611, 441, 99, 2, 30, "Input"],
Cell[11713, 445, 66, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11816, 451, 61, 1, 30, "Input"],
Cell[11880, 454, 66, 1, 29, "Output"]
}, Open  ]],
Cell[11961, 458, 256, 5, 90, "Input"],

Cell[CellGroupData[{
Cell[12242, 467, 93, 1, 30, "Input"],
Cell[12338, 470, 125, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12500, 477, 64, 1, 30, "Input"],
Cell[12567, 480, 125, 2, 48, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[12741, 488, 31, 0, 64, "Subtitle"],
Cell[12775, 490, 95, 2, 33, "Text"],
Cell[12873, 494, 88, 1, 30, "Input"],
Cell[12964, 497, 165, 2, 52, "Text"],
Cell[13132, 501, 177, 3, 70, "Input"],

Cell[CellGroupData[{
Cell[13334, 508, 52, 1, 30, "Input"],
Cell[13389, 511, 80, 1, 29, "Output"]
}, Open  ]],
Cell[13484, 515, 140, 3, 52, "Text"],
Cell[13627, 520, 92, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[13744, 525, 54, 1, 30, "Input"],
Cell[13801, 528, 1552, 20, 580, "Output"]
}, Open  ]],
Cell[15368, 551, 71, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[15464, 555, 51, 1, 30, "Input"],
Cell[15518, 558, 113, 2, 48, "Output"]
}, Open  ]],
Cell[15646, 563, 73, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[15744, 567, 140, 3, 30, "Input"],
Cell[15887, 572, 78, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16002, 578, 60, 1, 30, "Input"],
Cell[16065, 581, 37, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16139, 587, 52, 1, 30, "Input"],
Cell[16194, 590, 42, 1, 29, "Output"]
}, Open  ]],
Cell[16251, 594, 318, 5, 90, "Text"],
Cell[16572, 601, 551, 9, 190, "Input"],

Cell[CellGroupData[{
Cell[17148, 614, 52, 1, 30, "Input"],
Cell[17203, 617, 192, 4, 25, "Print"],
Cell[17398, 623, 58, 1, 25, "Print"],
Cell[17459, 626, 197, 4, 25, "Print"],
Cell[17659, 632, 111, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17807, 639, 39, 1, 30, "Input"],
Cell[17849, 642, 111, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17997, 649, 142, 3, 30, "Input"],
Cell[18142, 654, 75, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18254, 660, 60, 1, 30, "Input"],
Cell[18317, 663, 37, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18391, 669, 60, 1, 30, "Input"],
Cell[18454, 672, 192, 4, 25, "Print"],
Cell[18649, 678, 58, 1, 25, "Print"],
Cell[18710, 681, 197, 4, 25, "Print"],
Cell[18910, 687, 142, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19089, 694, 43, 1, 30, "Input"],
Cell[19135, 697, 138, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19310, 704, 126, 2, 30, "Input"],
Cell[19439, 708, 99, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19575, 715, 60, 1, 30, "Input"],
Cell[19638, 718, 37, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19712, 724, 91, 1, 30, "Input"],
Cell[19806, 727, 37, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19880, 733, 91, 1, 30, "Input"],
Cell[19974, 736, 37, 1, 29, "Output"]
}, Open  ]],
Cell[20026, 740, 386, 6, 90, "Text"],
Cell[20415, 748, 474, 10, 130, "Input"],
Cell[20892, 760, 353, 7, 110, "Input"],
Cell[21248, 769, 554, 9, 190, "Input"],

Cell[CellGroupData[{
Cell[21827, 782, 61, 1, 30, "Input"],
Cell[21891, 785, 192, 4, 25, "Print"],
Cell[22086, 791, 58, 1, 25, "Print"],
Cell[22147, 794, 197, 4, 25, "Print"],
Cell[22347, 800, 196, 3, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22580, 808, 92, 1, 50, "Input"],
Cell[22675, 811, 36, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22748, 817, 61, 1, 30, "Input"],
Cell[22812, 820, 192, 4, 25, "Print"],
Cell[23007, 826, 58, 1, 25, "Print"],
Cell[23068, 829, 197, 4, 25, "Print"],
Cell[23268, 835, 89, 1, 29, "Output"]
}, Open  ]],
Cell[23372, 839, 382, 7, 150, "Input"],
Cell[23757, 848, 154, 3, 51, "Input"],

Cell[CellGroupData[{
Cell[23936, 855, 65, 1, 30, "Input"],
Cell[24004, 858, 389, 6, 200, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24430, 869, 74, 1, 30, "Input"],
Cell[24507, 872, 3270, 42, 1169, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27814, 919, 53, 1, 30, "Input"],
Cell[27870, 922, 192, 4, 25, "Print"],
Cell[28065, 928, 58, 1, 25, "Print"],
Cell[28126, 931, 42, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

