(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      5846,        180]*)
(*NotebookOutlinePosition[      6497,        203]*)
(*  CellTagsIndexPosition[      6453,        199]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA - 2", "Title"],

Cell["Key Generation", "Subtitle"],

Cell["\<\
To start with we want to get a pair of random primes that are similar in \
size, but differ in length by several digits.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({p, q} = {GenerateStrongPrime[50], GenerateStrongPrime[52]}\)], "Input"],

Cell[BoxData[
    \({35413472432206057869802305829571325168050457462780721, 
      334737495079266665830925330562076455044964594859799}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The value of e should be coprime to (p-1)(q-1).  Simply letting e be a \
prime itself should be sufficient.  We will use our random prime generator \
using a smaller range.  The following value could be in the range ",
  Cell[BoxData[
      \(TraditionalForm\`10\^30\)]],
  " to ",
  Cell[BoxData[
      \(TraditionalForm\`10\^70\)]],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(e = GenerateRandomPrime[40]\)], "Input"],

Cell[BoxData[
    \(2103170878760351679801465009778409506967\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Now we must calculate the decryption exponent, ",
  StyleBox["d",
    FontSlant->"Italic"],
  ", as follows\n\td . e \[Congruent] 1 (mod \[Phi](n))"
}], "Text"],

Cell[BoxData[
    \(GetKey[e_, p_, q_] := 
      Module[{d, m, g, k}, \n\t\tm = \((p - 1)\)*\((q - 1)\); \n
        \t\t{g, {d, k}} = ExtendedGCD[e, m]; \n\t\t{e, d, p*q}]\)], "Input"],

Cell["\<\
Let's wrap this into a single function that given an approximate size for our \
random primes, generates p, q, and e and calculates d.\
\>", "Text"],

Cell[BoxData[
    \(GenerateKey[ord_, opts___] := 
      Module[{e, p, q}, \n\t\tp = GenerateStrongPrime[ord, opts]; \n\t\t
        q = GenerateStrongPrime[ord, opts]; \n\t\t
        e = GenerateRandomPrime[Floor[3/4*ord]]; \n\t\tGetKey[e, p, q]]\)], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GenerateKey[50]\)], "Input"],

Cell[BoxData[
    \({1063431120214172207743909826419886903, 
      \(-454110037708226419654328390929582407735871909061476765760311841992200\
9841421906075637940345757279341933\), 
      103659853829139466134023476031898375303199308937327898516091760383661891\
470403866838877142035223218365409}\)], "Output"]
}, Open  ]],

Cell["\<\
We finally wrap this up into a nice structure for key management.\
\>", "Text"],

Cell[BoxData[
    \(GenerateRSAKey[ord_, opts___] := 
      Module[{e, d, n}, \n\t\t{e, d, n} = GenerateKey[ord, opts]; \n\t\t
        RSAKey[e, d, n]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GenerateRSAKey[50]\)], "Input"],

Cell[BoxData[
    \(RSAKey[3784335200592959903172504038275773983, 
      \(-378207056249405353376300326159177003019231624547298173774391111671624\
7035632013181566241596380296130145\), 
      243001120612238014042138943925491903460163116794766340581666904900592678\
13615233521876199245794098999161]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Specifying the Exponent", "Subtitle"],

Cell["\<\
In this function we specify the encrypting exponent. Many applications \
require that a small exponent is used. This increases the speed at which \
encryption is performed by reducing the number of modular multiplications. \
Smart cards can use this method to create low exponent secret keys for \
signing and decryption. \
\>", "Text"],

Cell["\<\
The function has to be careful when choosing values for p and q. We must \
ensure that e has no common factors with p-1 or q-1. Hence the while loops. \
This may increase the time required to generate the key as several primes may \
be generated and tried. Note that one can specify the algorithm to use in \
choosing primes as in the other key generation functions.\
\>", "Text"],

Cell[BoxData[
    \(SpecifyExponent[ord_, e_, opts___] := \n\t
      Module[{p, q, d, n}, \n\t\tp = e + 1; q = e + 1; \n\t\t
        While[GCD[p - 1, e] > 1, \n\t\t\tp = GenerateStrongPrime[ord, opts]]; 
        \n\t\tWhile[GCD[q - 1, e] > 1, \n\t\t\t
          q = GenerateStrongPrime[ord, opts]]; \n\t\t
        d = PowerMod[e, \(-1\), p\ q]; \n\t\tRSAKey[e, d, p\ q]]\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{460, 569},
WindowMargins->{{44, Automatic}, {Automatic, -1}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 24, 0, 105, "Title"],
Cell[1758, 53, 34, 0, 64, "Subtitle"],
Cell[1795, 55, 144, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[1964, 62, 92, 1, 50, "Input"],
Cell[2059, 65, 149, 2, 67, "Output"]
}, Open  ]],
Cell[2223, 70, 371, 10, 71, "Text"],

Cell[CellGroupData[{
Cell[2619, 84, 60, 1, 30, "Input"],
Cell[2682, 87, 74, 1, 29, "Output"]
}, Open  ]],
Cell[2771, 91, 179, 5, 52, "Text"],
Cell[2953, 98, 184, 3, 90, "Input"],
Cell[3140, 103, 158, 3, 52, "Text"],
Cell[3301, 108, 262, 5, 110, "Input"],

Cell[CellGroupData[{
Cell[3588, 117, 48, 1, 30, "Input"],
Cell[3639, 120, 308, 5, 143, "Output"]
}, Open  ]],
Cell[3962, 128, 89, 2, 33, "Text"],
Cell[4054, 132, 164, 3, 70, "Input"],

Cell[CellGroupData[{
Cell[4243, 139, 51, 1, 30, "Input"],
Cell[4297, 142, 313, 5, 143, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4647, 152, 43, 0, 64, "Subtitle"],
Cell[4693, 154, 346, 6, 90, "Text"],
Cell[5042, 162, 390, 6, 109, "Text"],
Cell[5435, 170, 383, 6, 190, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

