(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     26702,        587]*)
(*NotebookOutlinePosition[     27352,        610]*)
(*  CellTagsIndexPosition[     27308,        606]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Representing Primes", "Title"],

Cell["\<\
A century ago, tables of prime numbers filled up numerous pages.  Today they \
are stored in computers in a more compact form.  We might try coding each odd \
number to be a 1 or 0 representing whether it is or isn't a prime.  For \
example the sequence 01110 means: 1 is not prime, 3 prime, 5 prime, 7 prime, \
9 not prime.  We can reduce this further by removing multiples of 5. The \
table of primes up to 50 could be given by\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(primesToFifty = {0110, 1111, 0101, 1010, 1110}\)], "Input"],

Cell[BoxData[
    \({110, 1111, 101, 1010, 1110}\)], "Output"]
}, Open  ]],

Cell["Each of these binary digits can be represented as hex digits", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(hex = {6, f, 5, a, e}\)], "Input"],

Cell[BoxData[
    \({6, f, 5, a, e}\)], "Output"]
}, Open  ]],

Cell["\<\
So we have compacted the table by realising that integers ending in certain \
digits (ending in even digits for example) can be excluded.  We excluded the \
end-digits 2, 4, 5, 6, 8 and 0 and could decide on the individual cases by \
assigning a 1 or 0.\
\>", "Text"],

Cell["\<\
Now consider doing this in another base, let's take base 3.  The numbers \
available to us are 0, 1, 2, 3, 4, and 5.  Again we can remove even numbers - \
these can never be primes.  We are left with 1, 3, and 5.  Of these three, \
only a number n having an end-digit of 3 will always be a multiple of 3 and \
therefore never be prime (for n>3 obviously since 3 is prime).\
\>", "Text"],

Cell["\<\
Let's try to show this by experimentation - just to see it working for real.  \
We will go back to the decimal system.  First generate a large enough set of \
primes to test on. Note that we will take only primes above the base (the 5th \
prime is 11, the 5000th prime is 48611).\
\>", "Text"],

Cell[BoxData[
    \(\(ps = Table[Prime[i], {i, 5, 5000}]; \)\)], "Input"],

Cell["\<\
The next step is to create a set of the last digits of all these primes.  The \
function IntegerDigits returns a list of the digits in a speicified base.  \
The Last function extracts the last digit of the number.  In this way we \
retrieve all the final digits of the primes represented under our chosen base \
- 10.\
\>", "Text"],

Cell[BoxData[
    \(\(digits = Map[Last[IntegerDigits[#, 10]]&, ps]; \)\)], "Input"],

Cell["The Union function now allows us to remove duplicates.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Union[digits, {}]\)], "Input"],

Cell[BoxData[
    \({1, 3, 7, 9}\)], "Output"]
}, Open  ]],

Cell["\<\
Let's use this technique get the digits that can appear at the end of a prime \
number given a certain base.  Note that we can wet the nth prime - i.e. the \
larger the more chance our system is accurate.\
\>", "Text"],

Cell[BoxData[
    \(EndDigits[base_, nthprime_:  5000] := 
      Module[{p = 1}, \n\t\tWhile[Prime[p] < base, p = p + 1]; \n\t\t
        Union[Map[Last[IntegerDigits[#, base]]&, 
            Table[Prime[i], {i, p, nthprime}]]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(EndDigits[10]\)], "Input"],

Cell[BoxData[
    \({1, 3, 7, 9}\)], "Output"]
}, Open  ]],

Cell["\<\
This means we can represent 10 digits with 4 bits (ignoring the first ten \
which can be stored in a table).\
\>", "Text"],

Cell[BoxData[
    \(Table[{i, EndDigits[i]}, {i, 6, 100}]\)], "Input"],

Cell["\<\
To save time, once I generated this table I stored it to avoid doing it again \
- quite time consuming.\
\>", "Text"],

Cell[BoxData[
    \(\(mytable = {{6, {1, 5}}, {7, {0, 1, 2, 3, 4, 5, 6}}, {
          8, {1, 3, 5, 7}}, {9, {1, 2, 4, 5, 7, 8}}, {10, {1, 3, 7, 9}}, {
          11, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10}}, {12, {1, 5, 7, 11}}, {
          13, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12}}, {
          14, {1, 3, 5, 9, 11, 13}}, {15, {1, 2, 4, 7, 8, 11, 13, 14}}, {
          16, {1, 3, 5, 7, 9, 11, 13, 15}}, {
          17, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16}}, {
          18, {1, 5, 7, 11, 13, 17}}, {
          19, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18}}, {20, {1, 3, 7, 9, 11, 13, 17, 19}}, {
          21, {1, 2, 4, 5, 8, 10, 11, 13, 16, 17, 19, 20}}, {
          22, {1, 3, 5, 7, 9, 13, 15, 17, 19, 21}}, {
          23, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22}}, {24, {1, 5, 7, 11, 13, 17, 19, 23}}, {
          25, {1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 16, 17, 18, 19, 21, 
            22, 23, 24}}, {26, {1, 3, 5, 7, 9, 11, 15, 17, 19, 21, 23, 25}}, {
          27, {1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 16, 17, 19, 20, 22, 23, 25, 
            26}}, {28, {1, 3, 5, 9, 11, 13, 15, 17, 19, 23, 25, 27}}, {
          29, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28}}, {
          30, {1, 7, 11, 13, 17, 19, 23, 29}}, {
          31, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30}}, {
          32, {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31}}, {
          33, {1, 2, 4, 5, 7, 8, 10, 13, 14, 16, 17, 19, 20, 23, 25, 26, 28, 
            29, 31, 32}}, {
          34, {1, 3, 5, 7, 9, 11, 13, 15, 19, 21, 23, 25, 27, 29, 31, 33}}, {
          35, {1, 2, 3, 4, 6, 8, 9, 11, 12, 13, 16, 17, 18, 19, 22, 23, 24, 
            26, 27, 29, 31, 32, 33, 34}}, {
          36, {1, 5, 7, 11, 13, 17, 19, 23, 25, 29, 31, 35}}, {
          37, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36}}, {
          38, {1, 3, 5, 7, 9, 11, 13, 15, 17, 21, 23, 25, 27, 29, 31, 33, 35, 
            37}}, {
          39, {1, 2, 4, 5, 7, 8, 10, 11, 14, 16, 17, 19, 20, 22, 23, 25, 28, 
            29, 31, 32, 34, 35, 37, 38}}, {
          40, {1, 3, 7, 9, 11, 13, 17, 19, 21, 23, 27, 29, 31, 33, 37, 39}}, {
          41, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40}}, {
          42, {1, 5, 11, 13, 17, 19, 23, 25, 29, 31, 37, 41}}, {
          43, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40, 41, 42}}, {
          44, {1, 3, 5, 7, 9, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 35, 37, 
            39, 41, 43}}, {
          45, {1, 2, 4, 7, 8, 11, 13, 14, 16, 17, 19, 22, 23, 26, 28, 29, 31, 
            32, 34, 37, 38, 41, 43, 44}}, {
          46, {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 25, 27, 29, 31, 33, 35, 
            37, 39, 41, 43, 45}}, {
          47, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46}}, {
          48, {1, 5, 7, 11, 13, 17, 19, 23, 25, 29, 31, 35, 37, 41, 43, 
            47}}, {49, {1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 15, 16, 17, 
            18, 19, 20, 22, 23, 24, 25, 26, 27, 29, 30, 31, 32, 33, 34, 36, 
            37, 38, 39, 40, 41, 43, 44, 45, 46, 47, 48}}, {
          50, {1, 3, 7, 9, 11, 13, 17, 19, 21, 23, 27, 29, 31, 33, 37, 39, 
            41, 43, 47, 49}}, {
          51, {1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 16, 19, 20, 22, 23, 25, 26, 
            28, 29, 31, 32, 35, 37, 38, 40, 41, 43, 44, 46, 47, 49, 50}}, {
          52, {1, 3, 5, 7, 9, 11, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 
            37, 41, 43, 45, 47, 49, 51}}, {
          53, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 
            50, 51, 52}}, {
          54, {1, 5, 7, 11, 13, 17, 19, 23, 25, 29, 31, 35, 37, 41, 43, 47, 
            49, 53}}, {
          55, {1, 2, 3, 4, 6, 7, 8, 9, 12, 13, 14, 16, 17, 18, 19, 21, 23, 
            24, 26, 27, 28, 29, 31, 32, 34, 36, 37, 38, 39, 41, 42, 43, 46, 
            47, 48, 49, 51, 52, 53, 54}}, {
          56, {1, 3, 5, 9, 11, 13, 15, 17, 19, 23, 25, 27, 29, 31, 33, 37, 
            39, 41, 43, 45, 47, 51, 53, 55}}, {
          57, {1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 16, 17, 20, 22, 23, 25, 26, 
            28, 29, 31, 32, 34, 35, 37, 40, 41, 43, 44, 46, 47, 49, 50, 52, 
            53, 55, 56}}, {
          58, {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 31, 33, 35, 
            37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57}}, {
          59, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 
            50, 51, 52, 53, 54, 55, 56, 57, 58}}, {
          60, {1, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 49, 53, 
            59}}, {61, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 
            16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 
            32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 
            48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60}}, {
          62, {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 33, 35, 
            37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61}}, {
          63, {1, 2, 4, 5, 8, 10, 11, 13, 16, 17, 19, 20, 22, 23, 25, 26, 29, 
            31, 32, 34, 37, 38, 40, 41, 43, 44, 46, 47, 50, 52, 53, 55, 58, 
            59, 61, 62}}, {
          64, {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 
            35, 37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63}}, {
          65, {1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 14, 16, 17, 18, 19, 21, 22, 
            23, 24, 27, 28, 29, 31, 32, 33, 34, 36, 37, 38, 41, 42, 43, 44, 
            46, 47, 48, 49, 51, 53, 54, 56, 57, 58, 59, 61, 62, 63, 64}}, {
          66, {1, 5, 7, 13, 17, 19, 23, 25, 29, 31, 35, 37, 41, 43, 47, 49, 
            53, 59, 61, 65}}, {
          67, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 
            50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 
            66}}, {68, {1, 3, 5, 7, 9, 11, 13, 15, 19, 21, 23, 25, 27, 29, 
            31, 33, 35, 37, 39, 41, 43, 45, 47, 49, 53, 55, 57, 59, 61, 63, 
            65, 67}}, {
          69, {1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 16, 17, 19, 20, 22, 25, 26, 
            28, 29, 31, 32, 34, 35, 37, 38, 40, 41, 43, 44, 47, 49, 50, 52, 
            53, 55, 56, 58, 59, 61, 62, 64, 65, 67, 68}}, {
          70, {1, 3, 9, 11, 13, 17, 19, 23, 27, 29, 31, 33, 37, 39, 41, 43, 
            47, 51, 53, 57, 59, 61, 67, 69}}, {
          71, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 
            50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 
            66, 67, 68, 69, 70}}, {
          72, {1, 5, 7, 11, 13, 17, 19, 23, 25, 29, 31, 35, 37, 41, 43, 47, 
            49, 53, 55, 59, 61, 65, 67, 71}}, {
          73, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 
            50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 
            66, 67, 68, 69, 70, 71, 72}}, {
          74, {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 
            35, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 
            69, 71, 73}}, {
          75, {1, 2, 4, 7, 8, 11, 13, 14, 16, 17, 19, 22, 23, 26, 28, 29, 31, 
            32, 34, 37, 38, 41, 43, 44, 46, 47, 49, 52, 53, 56, 58, 59, 61, 
            62, 64, 67, 68, 71, 73, 74}}, {
          76, {1, 3, 5, 7, 9, 11, 13, 15, 17, 21, 23, 25, 27, 29, 31, 33, 35, 
            37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 59, 61, 63, 65, 67, 69, 
            71, 73, 75}}, {
          77, {1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 13, 15, 16, 17, 18, 19, 20, 
            23, 24, 25, 26, 27, 29, 30, 31, 32, 34, 36, 37, 38, 39, 40, 41, 
            43, 45, 46, 47, 48, 50, 51, 52, 53, 54, 57, 58, 59, 60, 61, 62, 
            64, 65, 67, 68, 69, 71, 72, 73, 74, 75, 76}}, {
          78, {1, 5, 7, 11, 17, 19, 23, 25, 29, 31, 35, 37, 41, 43, 47, 49, 
            53, 55, 59, 61, 67, 71, 73, 77}}, {
          79, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 
            50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 
            66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78}}, {
          80, {1, 3, 7, 9, 11, 13, 17, 19, 21, 23, 27, 29, 31, 33, 37, 39, 
            41, 43, 47, 49, 51, 53, 57, 59, 61, 63, 67, 69, 71, 73, 77, 
            79}}, {81, {1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 16, 17, 19, 20, 22, 
            23, 25, 26, 28, 29, 31, 32, 34, 35, 37, 38, 40, 41, 43, 44, 46, 
            47, 49, 50, 52, 53, 55, 56, 58, 59, 61, 62, 64, 65, 67, 68, 70, 
            71, 73, 74, 76, 77, 79, 80}}, {
          82, {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 
            35, 37, 39, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 
            69, 71, 73, 75, 77, 79, 81}}, {
          83, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 
            50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 
            66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 
            82}}, {84, {1, 5, 11, 13, 17, 19, 23, 25, 29, 31, 37, 41, 43, 47, 
            53, 55, 59, 61, 65, 67, 71, 73, 79, 83}}, {
          85, {1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 16, 18, 19, 21, 22, 
            23, 24, 26, 27, 28, 29, 31, 32, 33, 36, 37, 38, 39, 41, 42, 43, 
            44, 46, 47, 48, 49, 52, 53, 54, 56, 57, 58, 59, 61, 62, 63, 64, 
            66, 67, 69, 71, 72, 73, 74, 76, 77, 78, 79, 81, 82, 83, 84}}, {
          86, {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 
            35, 37, 39, 41, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 
            69, 71, 73, 75, 77, 79, 81, 83, 85}}, {
          87, {1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 16, 17, 19, 20, 22, 23, 25, 
            26, 28, 31, 32, 34, 35, 37, 38, 40, 41, 43, 44, 46, 47, 49, 50, 
            52, 53, 55, 56, 59, 61, 62, 64, 65, 67, 68, 70, 71, 73, 74, 76, 
            77, 79, 80, 82, 83, 85, 86}}, {
          88, {1, 3, 5, 7, 9, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 35, 37, 
            39, 41, 43, 45, 47, 49, 51, 53, 57, 59, 61, 63, 65, 67, 69, 71, 
            73, 75, 79, 81, 83, 85, 87}}, {
          89, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 
            18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 
            34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 
            50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 
            66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 
            82, 83, 84, 85, 86, 87, 88}}, {
          90, {1, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 49, 53, 59, 
            61, 67, 71, 73, 77, 79, 83, 89}}, {
          91, {1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 15, 16, 17, 18, 19, 20, 
            22, 23, 24, 25, 27, 29, 30, 31, 32, 33, 34, 36, 37, 38, 40, 41, 
            43, 44, 45, 46, 47, 48, 50, 51, 53, 54, 55, 57, 58, 59, 60, 61, 
            62, 64, 66, 67, 68, 69, 71, 72, 73, 74, 75, 76, 79, 80, 81, 82, 
            83, 85, 86, 87, 88, 89, 90}}, {
          92, {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 25, 27, 29, 31, 33, 35, 
            37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 
            71, 73, 75, 77, 79, 81, 83, 85, 87, 89, 91}}, {
          93, {1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 16, 17, 19, 20, 22, 23, 25, 
            26, 28, 29, 32, 34, 35, 37, 38, 40, 41, 43, 44, 46, 47, 49, 50, 
            52, 53, 55, 56, 58, 59, 61, 64, 65, 67, 68, 70, 71, 73, 74, 76, 
            77, 79, 80, 82, 83, 85, 86, 88, 89, 91, 92}}, {
          94, {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 
            35, 37, 39, 41, 43, 45, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 
            69, 71, 73, 75, 77, 79, 81, 83, 85, 87, 89, 91, 93}}, {
          95, {1, 2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14, 16, 17, 18, 21, 22, 
            23, 24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 39, 41, 42, 43, 
            44, 46, 47, 48, 49, 51, 52, 53, 54, 56, 58, 59, 61, 62, 63, 64, 
            66, 67, 68, 69, 71, 72, 73, 74, 77, 78, 79, 81, 82, 83, 84, 86, 
            87, 88, 89, 91, 92, 93, 94}}, {
          96, {1, 5, 7, 11, 13, 17, 19, 23, 25, 29, 31, 35, 37, 41, 43, 47, 
            49, 53, 55, 59, 61, 65, 67, 71, 73, 77, 79, 83, 85, 89, 91, 
            95}}, {97, {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 
            16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 
            32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 
            48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 
            64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 
            80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 
            96}}, {98, {1, 3, 5, 9, 11, 13, 15, 17, 19, 23, 25, 27, 29, 31, 
            33, 37, 39, 41, 43, 45, 47, 51, 53, 55, 57, 59, 61, 65, 67, 69, 
            71, 73, 75, 79, 81, 83, 85, 87, 89, 93, 95, 97}}, {
          99, {1, 2, 4, 5, 7, 8, 10, 13, 14, 16, 17, 19, 20, 23, 25, 26, 28, 
            29, 31, 32, 34, 35, 37, 38, 40, 41, 43, 46, 47, 49, 50, 52, 53, 
            56, 58, 59, 61, 62, 64, 65, 67, 68, 70, 71, 73, 74, 76, 79, 80, 
            82, 83, 85, 86, 89, 91, 92, 94, 95, 97, 98}}, {
          100, {1, 3, 7, 9, 11, 13, 17, 19, 21, 23, 27, 29, 31, 33, 37, 39, 
            41, 43, 47, 49, 51, 53, 57, 59, 61, 63, 67, 69, 71, 73, 77, 79, 
            81, 83, 87, 89, 91, 93, 97, 99}}}; \)\)], "Input"],

Cell["\<\
This function takes an entry from the following table and returns:
\tthe base,
\tthe number of possible prime end-digits
\tthe percentage of possible prime digits\
\>", "Text"],

Cell[BoxData[
    \(AnalyseEntry[entry_] := 
      Module[{b, len}, \n\t\t
        b = entry\[LeftDoubleBracket]1\[RightDoubleBracket]; \n\t\t
        len = Length[entry\[LeftDoubleBracket]2\[RightDoubleBracket]]; \n
        \t\t{b, len, Divide[len, b]}]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(AnalyseEntry[mytable\[LeftDoubleBracket]1\[RightDoubleBracket]]\)], 
  "Input"],

Cell[BoxData[
    \({6, 2, 1\/3}\)], "Output"]
}, Open  ]],

Cell["\<\
Analyse the entire table.  Output suppressed until it's sorted.\
\>", "Text"],

Cell[BoxData[
    \(\(anal = Map[AnalyseEntry, mytable]; \)\)], "Input"],

Cell["Let's do some ordering on the table", "Text"],

Cell[BoxData[
    \(mySort[entry1_, entry2_] := 
      Greater[entry1\[LeftDoubleBracket]3\[RightDoubleBracket], 
        entry2\[LeftDoubleBracket]3\[RightDoubleBracket]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sort[anal, mySort]\)], "Input"],

Cell[BoxData[
    \({{97, 97, 1}, {89, 89, 1}, {83, 83, 1}, {79, 79, 1}, {73, 73, 1}, {71, 
        71, 1}, {67, 67, 1}, {61, 61, 1}, {59, 59, 1}, {53, 53, 1}, {47, 47, 
        1}, {43, 43, 1}, {41, 41, 1}, {37, 37, 1}, {31, 31, 1}, {29, 29, 1}, {
        23, 23, 1}, {19, 19, 1}, {17, 17, 1}, {13, 13, 1}, {11, 11, 1}, {7, 
        7, 1}, {49, 42, 6\/7}, {25, 20, 4\/5}, {91, 72, 72\/91}, {77, 60, 
        60\/77}, {95, 72, 72\/95}, {85, 64, 64\/85}, {65, 48, 48\/65}, {55, 
        40, 8\/11}, {35, 24, 24\/35}, {81, 54, 2\/3}, {27, 18, 2\/3}, {9, 6, 
        2\/3}, {93, 60, 20\/31}, {87, 56, 56\/87}, {69, 44, 44\/69}, {57, 36, 
        12\/19}, {51, 32, 32\/51}, {39, 24, 8\/13}, {99, 60, 20\/33}, {33, 
        20, 20\/33}, {63, 36, 4\/7}, {21, 12, 4\/7}, {75, 40, 8\/15}, {45, 
        24, 8\/15}, {15, 8, 8\/15}, {64, 32, 1\/2}, {32, 16, 1\/2}, {16, 8, 
        1\/2}, {8, 4, 1\/2}, {94, 46, 23\/47}, {86, 42, 21\/43}, {82, 40, 
        20\/41}, {74, 36, 18\/37}, {62, 30, 15\/31}, {58, 28, 14\/29}, {92, 
        44, 11\/23}, {46, 22, 11\/23}, {76, 36, 9\/19}, {38, 18, 9\/19}, {68, 
        32, 8\/17}, {34, 16, 8\/17}, {52, 24, 6\/13}, {26, 12, 6\/13}, {88, 
        40, 5\/11}, {44, 20, 5\/11}, {22, 10, 5\/11}, {98, 42, 3\/7}, {56, 
        24, 3\/7}, {28, 12, 3\/7}, {14, 6, 3\/7}, {100, 40, 2\/5}, {80, 32, 
        2\/5}, {50, 20, 2\/5}, {40, 16, 2\/5}, {20, 8, 2\/5}, {10, 4, 2\/5}, {
        70, 24, 12\/35}, {96, 32, 1\/3}, {72, 24, 1\/3}, {54, 18, 1\/3}, {48, 
        16, 1\/3}, {36, 12, 1\/3}, {24, 8, 1\/3}, {18, 6, 1\/3}, {12, 4, 
        1\/3}, {6, 2, 1\/3}, {78, 24, 4\/13}, {66, 20, 10\/33}, {84, 24, 
        2\/7}, {42, 12, 2\/7}, {90, 24, 4\/15}, {60, 16, 4\/15}, {30, 8, 
        4\/15}}\)], "Output"]
}, Open  ]],

Cell["\<\
This helps us to see which base provides us with the least encoding overhead. \
 Note that for prime bases are extremely poor, while the bases that have many \
different prime factors are the best.  The best in our range was 30 = 2 * 3 * \
5.  We should see if 210 = 2 * 3 * 5 * 7 is any better:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(chk210 = EndDigits[210]\)], "Input"],

Cell[BoxData[
    \({1, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 
      79, 83, 89, 97, 101, 103, 107, 109, 113, 121, 127, 131, 137, 139, 143, 
      149, 151, 157, 163, 167, 169, 173, 179, 181, 187, 191, 193, 197, 199, 
      209}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(AnalyseEntry[{210, chk210}]\)], "Input"],

Cell[BoxData[
    \({210, 48, 8\/35}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Greater[8/35, 4/14]\)], "Input"],

Cell[BoxData[
    \(False\)], "Output"]
}, Open  ]],

Cell["So we see that 210 is even better than 30.  What about", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(2*3*5*7*11\)], "Input"],

Cell[BoxData[
    \(2310\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(AnalyseEntry[{2310, EndDigits[2310, 10000]}]\)], "Input"],

Cell[BoxData[
    \({2310, 480, 16\/77}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Greater[16/77, 8/35]\)], "Input"],

Cell[BoxData[
    \(False\)], "Output"]
}, Open  ]],

Cell["That is even better.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(EulerPhi[210]\)], "Input"],

Cell[BoxData[
    \(48\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(EulerPhi[30]\)], "Input"],

Cell[BoxData[
    \(8\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(EulerPhi[2310]\)], "Input"],

Cell[BoxData[
    \(480\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(EulerPhi[2310*12]\)], "Input"],

Cell[BoxData[
    \(5760\)], "Output"]
}, Open  ]],

Cell["\<\
Well there you have it, all the work in this notebook could very easily have \
been done in a line or two.  The EulerPhi function returns the number of less \
than n, which are coprime to n, when n is the argument.  It makes sense that \
if the end-digit is not coprime to the base, (the base and digit can be \
divided by the same number) then the end-digit signifies that the number is \
definately not prime.  We use this every day in the decimal system to \
calculate the factors of a number.  We can divide by 2 if a number is even or \
5 if it ends with 0 or 5.  \
\>", "Text"],

Cell["\<\
On the other hand, when an end-digit is coprime to the base, like a number \
counted by EulerPhi, it possibly does not have any factors, which means that \
it might be a prime.  What this notebook has tried to discover is if one base \
representation is better than another.  And we have decided that yes this is \
true.  The number of bits needed to encode 30 primes would be 8 using base 30 \
representation.  210 primes can be represented with 48 bits, and so on.  The \
conveniant thing about this is that we have multiples of 8 which map nicely \
to hex and therefore are machine useable.\
\>", "Text"],

Cell["\<\
I think I will look at writing some functions for base 30 as it is easier to \
handle.  It will show the typical storage needed.\
\>", "Text"]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{641, 673},
WindowMargins->{{71, Automatic}, {Automatic, 2}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 36, 0, 105, "Title"],
Cell[1770, 53, 453, 7, 90, "Text"],

Cell[CellGroupData[{
Cell[2248, 64, 79, 1, 30, "Input"],
Cell[2330, 67, 62, 1, 29, "Output"]
}, Open  ]],
Cell[2407, 71, 76, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[2508, 75, 54, 1, 30, "Input"],
Cell[2565, 78, 49, 1, 29, "Output"]
}, Open  ]],
Cell[2629, 82, 277, 5, 52, "Text"],
Cell[2909, 89, 396, 6, 71, "Text"],
Cell[3308, 97, 303, 5, 71, "Text"],
Cell[3614, 104, 73, 1, 30, "Input"],
Cell[3690, 107, 341, 6, 71, "Text"],
Cell[4034, 115, 84, 1, 30, "Input"],
Cell[4121, 118, 70, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[4216, 122, 50, 1, 30, "Input"],
Cell[4269, 125, 46, 1, 29, "Output"]
}, Open  ]],
Cell[4330, 129, 228, 4, 52, "Text"],
Cell[4561, 135, 240, 4, 70, "Input"],

Cell[CellGroupData[{
Cell[4826, 143, 46, 1, 30, "Input"],
Cell[4875, 146, 46, 1, 29, "Output"]
}, Open  ]],
Cell[4936, 150, 132, 3, 33, "Text"],
Cell[5071, 155, 70, 1, 30, "Input"],
Cell[5144, 158, 127, 3, 33, "Text"],
Cell[5274, 163, 15166, 221, 3310, "Input"],
Cell[20443, 386, 186, 5, 90, "Text"],
Cell[20632, 393, 267, 5, 90, "Input"],

Cell[CellGroupData[{
Cell[20924, 402, 99, 2, 30, "Input"],
Cell[21026, 406, 46, 1, 42, "Output"]
}, Open  ]],
Cell[21087, 410, 87, 2, 33, "Text"],
Cell[21177, 414, 72, 1, 30, "Input"],
Cell[21252, 417, 51, 0, 33, "Text"],
Cell[21306, 419, 184, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[21515, 426, 51, 1, 30, "Input"],
Cell[21569, 429, 1734, 23, 547, "Output"]
}, Open  ]],
Cell[23318, 455, 319, 5, 71, "Text"],

Cell[CellGroupData[{
Cell[23662, 464, 56, 1, 30, "Input"],
Cell[23721, 467, 272, 4, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24030, 476, 60, 1, 30, "Input"],
Cell[24093, 479, 50, 1, 42, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24180, 485, 52, 1, 30, "Input"],
Cell[24235, 488, 39, 1, 29, "Output"]
}, Open  ]],
Cell[24289, 492, 70, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[24384, 496, 43, 1, 30, "Input"],
Cell[24430, 499, 38, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24505, 505, 77, 1, 30, "Input"],
Cell[24585, 508, 53, 1, 42, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24675, 514, 53, 1, 30, "Input"],
Cell[24731, 517, 39, 1, 29, "Output"]
}, Open  ]],
Cell[24785, 521, 36, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[24846, 525, 46, 1, 30, "Input"],
Cell[24895, 528, 36, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24968, 534, 45, 1, 30, "Input"],
Cell[25016, 537, 35, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25088, 543, 47, 1, 30, "Input"],
Cell[25138, 546, 37, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25212, 552, 50, 1, 30, "Input"],
Cell[25265, 555, 38, 1, 29, "Output"]
}, Open  ]],
Cell[25318, 559, 593, 9, 109, "Text"],
Cell[25914, 570, 617, 9, 109, "Text"],
Cell[26534, 581, 152, 3, 33, "Text"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

